/*
Copyright (C) 2003 - Derek John Evans 

This file is part of Yeti3D Portable Engine

Yeti3D is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Original Source: 2003 - Derek J. Evans <derek@theteahouse.com.au>
Prepared for public release: 10/24/2003 - Derek J. Evans <derek@theteahouse.com.au>
*/

/*
**  This demo using SDL was made by OneThirty8, but is basically a
**  modification of the DirectDraw demo by Derek J. Evans.  The idea here
**  is to do basically the same thing, but also to be portable to
**  Mac OSX and Linux.  Surprisingly, this runs fairly well on the
**  Dreamcast without using the 3D hardware.
*/

/*
Port: Yeti3D para Wiz.

author: JDURANMASTER
version: 1.0

**  notas: este es un port del motor Yeti3D de GBA realizado en SDL 
**         para la consola GP2X Wiz.
*/

#include <stdlib.h>
#include <stdio.h>
#include <SDL/SDL.h>
#include <dirent.h>
#include <unistd.h>
#include "caanookeys.h"
#include "game.h"

framebuffer_t framebuffer; //gives us somewhere to draw our stuff.
yeti_t yeti;
int oldTick = 0;		// Used to help limit the framerate.
int done = 0;			// Keeps track of whether or not we're still playing.
SDL_Surface *screen;	// our video surface
SDL_Event event;
//estrutura para manejar el joystick
SDL_Joystick* caanooController;
//numero de botones del joystick
int numButtons;

void sdl3d_flip() // checks whether we're ready to draw a frame, and if so it draws it.
{
	int currentTick;
	int waitTicks;
    
	currentTick=SDL_GetTicks();

	oldTick += YETI_VIEWPORT_INTERVAL;
	waitTicks = (oldTick + (YETI_VIEWPORT_INTERVAL)) - currentTick;
	oldTick = currentTick;

	/* Only loop through the game loop and draw if enough
	   time has passed since the last frame.
	   The viewport_to_video function takes the frame we've drawm to
	   our 15-bpp 'framebuffer' in memory, converts it to 16-bit,
	   and throws it onto our actual video surface.  Look at yeti.c
	   to see what is going on under the hood here. */
	if(waitTicks>0)
	{
		SDL_Delay(waitTicks);
	}

	game_loop(&yeti);
    
	viewport_to_video( (rgb555_t*)screen->pixels, screen->pitch, &yeti.viewport, 0xf800, 0x07e0, 0x001f);
	
	SDL_Flip(screen);
}



int main(int argc, char *argv[])
{
	Uint8* keys;  // to keep track of keypresses later on.

	//Inicializamos el video, el tijmer, y el joystick.
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_JOYSTICK) < 0 )
	{
		printf("Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);
	SDL_ShowCursor(0);
	caanooController=SDL_JoystickOpen(0);
	//obtenemos el numero de botones del joystick
	numButtons = SDL_JoystickNumButtons(caanooController);
	//permitimos los eventos del joystick
	SDL_JoystickEventState(SDL_ENABLE);
	
	// set up our video surface to have the same width and height as our viewport
	screen = SDL_SetVideoMode(YETI_VIEWPORT_WIDTH, YETI_VIEWPORT_HEIGHT, 16, SDL_SWSURFACE);
	if ( screen == NULL )
	{
		printf("Unable to set %d, %d 16-bit video: %s\n",YETI_VIEWPORT_WIDTH,
               YETI_VIEWPORT_HEIGHT, SDL_GetError());
		exit(1);
	}
  
	//Initiallize Yeti3D
	yeti_init(&yeti, &framebuffer, &framebuffer, textures, palette, lua);
	game_init(&yeti);
    
	while(done==0){              
		sdl3d_flip();
		keys = SDL_GetKeyState(NULL);
		if ( SDL_PollEvent(&event) ) {
			switch (event.type) {
			case SDL_JOYAXISMOTION:
				switch(event.jaxis.axis) {
				case 0:
					if(event.jaxis.value<-16000) {
						yeti.keyboard.left = 1;
						yeti.keyboard.right = 0;
					} else if(event.jaxis.value>16000) {
						yeti.keyboard.left = 0;
						yeti.keyboard.right = 1;
					} else {
						yeti.keyboard.left = 0;
						yeti.keyboard.right = 0;
					}
					break;
				case 1:
                                       if(event.jaxis.value<-16000) {
                                                yeti.keyboard.up = 1;
                                                yeti.keyboard.down = 0;
                                        } else if(event.jaxis.value>16000) {
                                                yeti.keyboard.up = 0;
                                                yeti.keyboard.down = 1;
                                        } else {
                                                yeti.keyboard.up = 0;
                                                yeti.keyboard.down = 0;
                                        }
					break;
				}
				break;
			}
			
	         }
		//yeti.keyboard.up     = SDL_JoystickGetButton(caanooController,CAANOO_UP);//WIZ_UP;//keys[SDLK_UP];
		//yeti.keyboard.down   = SDL_JoystickGetButton(caanooController,CAANOO_DOWN);//WIZ_DOWN;//keys[SDLK_DOWN];
		//yeti.keyboard.left   = SDL_JoystickGetButton(caanooController,CAANOO_LEFT);//WIZ_LEFT;//keys[SDLK_LEFT];
		//yeti.keyboard.right  = SDL_JoystickGetButton(caanooController,CAANOO_RIGHT);//WIZ_RIGHT;//keys[SDLK_RIGHT];
		yeti.keyboard.a      = SDL_JoystickGetButton(caanooController,CAANOO_B);//WIZ_B;//keys[SDLK_RETURN];
		yeti.keyboard.b      = SDL_JoystickGetButton(caanooController,CAANOO_X);//WIZ_X;//keys[SDLK_SPACE];
		yeti.keyboard.l      = SDL_JoystickGetButton(caanooController,CAANOO_L);//WIZ_R;//keys[SDLK_a];
		yeti.keyboard.r      = SDL_JoystickGetButton(caanooController,CAANOO_R);//WIZ_L;//keys[SDLK_y];
		yeti.keyboard.select = SDL_JoystickGetButton(caanooController,CAANOO_SELECT);//WIZ_SELECT;//keys[SDLK_ESCAPE];
		done = SDL_JoystickGetButton(caanooController,CAANOO_HOME);
	}
	
	//cierre ordenado
	SDL_JoystickClose(caanooController);
    SDL_Quit();
    chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);

	return 0;
}









